module ActsAsMarkupOn
  class Railtie < Rails::Railtie
    config.acts_as_markup_on = ActiveSupport::OrderedOptions.new
    
    initializer 'acts_as_markup_on.set_config', :after => 'active_record.initialize_database' do |app|
      ActiveSupport.on_load(:acts_as_markup_on) do
        self.markdown_library = app.config.acts_as_markup_on.markdown_library
        self.mediawiki_library = app.config.acts_as_markup_on.mediawiki_library
      end
    end
    
    initializer 'acts_as_markup_on.extend_active_record', :after => 'acts_as_markup_on.set_config' do |app|
      ActiveSupport.on_load(:active_record) do
        require 'acts_as_markup_on/exts/object'
        require 'acts_as_markup_on/stringlike'
        require 'acts_as_markup_on/active_record_extension'
        self.send :include, ActsAsMarkupOn::ActiveRecordExtension
      end
    end
    
    config.before_configuration do
      config.acts_as_markup_on['markdown_library'] ||= :rdiscount
      config.acts_as_markup_on['mediawiki_library'] ||= :wikicloth
    end
  end
end
